require 'fastlane/action'
require_relative '../helper/ios_cd_helper'

module Fastlane
  module Actions
    class BuildAndDeployAction < Action
      def self.run(params)
        # Check parameters
        unless Helper::IosCdHelper.is_set(params[:beta_type])
          UI.error("❌ Parameters beta_type cannot be null")
          puts("Error on beta type parameter")
        end

        UI.message("⌛️ Building and deploying to Store in #{params[:beta_type]}..")

        # Decrypt the keys archive and Extract the keys archive
        Helper::IosCdHelper.decrypt_ios_keys('.')
        UI.message("👉🏼 Credentials decrypted.")

        # Retrieve credentials
        creds = Helper::IosCdHelper.parseIosCredentials('.')
        UI.message("👉🏼 Credentials parsed.")

        # Delete keychain if existing
        if File.exist?(File.expand_path("~/Library/Keychains/#{name}-db"))
          other_action.delete_keychain(
            name: creds['temp_keychain_user'].to_s
          )
        end

        # Create keychain to store certifs
        other_action.create_keychain(
          name: creds['temp_keychain_user'].to_s,
          password:  creds['temp_keychain_password'].to_s,
          unlock: false,
          timeout: 0
        )
        UI.message("👉🏼 New keychain created")

        # Obtain App Store Connect API key
        api_key = other_action.app_store_connect_api_key(
          key_id: creds['apple_key_id'].to_s,
          issuer_id: creds['apple_issuer_id'].to_s,
          key_content: creds['apple_key_content'].to_s,
          duration: 1200,
          in_house: false
        )
        UI.message("👉🏼 API Key formated")

        last_testflight_build_number =
          other_action.latest_testflight_build_number(
            api_key: api_key,
            team_id: creds['team_id'].to_s,
            team_name: creds['team_name'].to_s,
            platform: 'ios',
            app_identifier: creds['developer_app_identifier'].to_s,
            username: creds['username'].to_s
          ) + 1

        # Increment build number for latest TestFlight build
        other_action.increment_build_number(
          build_number: last_testflight_build_number + 1,
          xcodeproj: "Runner.xcodeproj"
        )
        UI.message("👉🏼 Build number incremented")

        # Install Cocoapods
        other_action.cocoapods(
          clean_install: true,
          clean: true,
          integrate: true,
          podfile: "./Podfile"
        )
        UI.message("👉🏼 Pod got")

        # Set up code signing using match
        # Configures and runs `match` which manages code signing certificates and provisioning profiles for the project.
        # The function takes the app's bundle identifier, an authorization token for the project's Git repository, and the name and password for a temporary keychain used to store the signing certificate.
        # It uses the App Store Connect API key to access the App Store and increment the build number.
        # It then runs `gym` to build and sign the app using the selected provisioning profile, and finally, uses `pilot` to upload the app to TestFlight for beta testing.
        other_action.match(
          api_key: api_key,
          type: 'appstore',
          app_identifier: creds['app_identifier_extensions'],
          git_basic_authorization: Base64.strict_encode64(ENV["GIT_AUTHORIZATION"]),
          keychain_name: creds['temp_keychain_user'].to_s,
          keychain_password: creds['temp_keychain_password'].to_s,
          git_url: creds['git_url'].to_s,
          username:  creds['username'].to_s,
          team_id:  creds['team_id'].to_s,
          team_name: creds['team_name'].to_s,
          git_url: creds['git_url'].to_s,
          storage_mode: "git"
        )
        UI.message("👉🏼 App signed")

        # Build and export app using Gym
        # Builds and packages an iOS app or framework for distribution to the App Store, TestFlight, or Enterprise distribution.
        other_action.gym(
          configuration: "Release",
          workspace: "Runner.xcworkspace",
          export_method: "app-store",
          export_options: {
            provisioningProfiles: creds['provisioning_profiles']
          }
        )
        UI.message("👉🏼 App built")

        # Upload build to App Store Connect using Pilot
        other_action.pilot(
          api_key: api_key,
          apple_id: creds['developer_app_id'].to_s,
          app_identifier: creds['developer_app_identifier'].to_s,
          skip_waiting_for_build_processing: true,
          skip_submission: true,
          distribute_external: false,
          notify_external_testers: false,
          ipa: "./Runner.ipa"
        )

        # Delete keychain if existing
        if File.exist?(File.expand_path("~/Library/Keychains/#{name}-db"))
          other_action.delete_keychain(
            name: creds['temp_keychain_user']
          )
        end
      end

      def self.description
        "Testflight and AppStore deployment plugin for Fastlane, simplifying the  build and deployment porcess to internal, external, and production channels, and promoting builds for testing."
      end

      def self.authors
        ["SAS Wyatt Studio"]
      end

      def self.return_value
      end

      def self.details
        "The Fastlane Testflight / Appstore deloyment action streamlines the build and deployment to internal, external and production channels. Allow you to promote builds on testflight beta tests."
      end

      def self.available_options
        [
          FastlaneCore::ConfigItem.new(key: :beta_type,
            env_name: "IOS_CD_TEST_TYPE",
            optional: false,
            description: "Type of test (internal, external, production)")
        ]
      end

      def self.is_supported?(platform)
        [:ios].include?(platform)
      end
    end
  end
end
